@@ -24,6 +24,9 @@
 #' This allows for the avoidance of variables such spike-in RNAs, sample IDs, sample acquisition dates, etc. which are not relevant for outcome prediction.
 #' @param maxMissingProp Default: 0.0. A proportion less than 1 which is the maximum
 #' tolerated proportion of missingness for a feature to be retained for modelling.
+#' @param maxSimilarity Default: 1. A number between 0 and 1 which is the maximum similarity between a pair of
+#' variables to be both kept in the data set. For numerical variables, the Pearson correlation is used and for categorical variables,
+#' the Chi-squared test p-value is used. For a pair that is too similar, the second variable will be excluded from the data set.
 #' @param topNvariance Default: NULL. If \code{measurements} is a \code{MultiAssayExperiment} or list of tabular data, a named integer vector of most variable 
 #' features per assay to subset to. If the input data is a single table, then simply a single integer.
 #' If an assays has less features, it won't be reduced in size but stay as-is.
@@ -55,10 +58,11 @@ setMethod("prepareData", "data.frame",
   prepareData(S4Vectors::DataFrame(measurements, check.names = FALSE), outcome, ...)
 })
 
+#' @importFrom dcanr cor.pairs
 #' @rdname prepareData
 #' @export
 setMethod("prepareData", "DataFrame",
-  function(measurements, outcome, useFeatures = NULL, maxMissingProp = 0.0, topNvariance = NULL)
+  function(measurements, outcome, useFeatures = NULL, maxMissingProp = 0.0, maxSimilarity = 1, topNvariance = NULL)
 {
   if(is.null(rownames(measurements)))
   {
@@ -238,6 +242,30 @@ setMethod("prepareData", "DataFrame",
     }))
   }
   
+  if(maxSimilarity < 1)
+  {
+    categoricalFeatures <- sapply(measurements, class) %in% c("factor", "character")
+    if(any(categoricalFeatures))
+    {
+      checkPairs <- combn(which(categoricalFeatures), 2)
+      pValues <- apply(checkPairs, 2, function(checkPair)
+      {
+        fisher.test(table(measurements[, checkPair[1]], measurements[, checkPair[2]]))$p.value
+      })
+      if(any(pValues) < maxSimilarity) dropFeatures <- checkPairs[2, which(pValues < maxSimilarity)]
+    }
+    numericFeatures <- sapply(measurements, class) == "numeric"
+    if(any(numericFeatures))
+    {
+        correlations <- dcanr::cor.pairs(as.matrix(measurements[, numericFeatures]))
+        diag(correlations) <- 0
+        dropFeatures <- c(dropFeatures, unique(which(correlations > maxSimilarity, arr.ind = TRUE)[, 2]))
+    }
+    dropFeatures <- character()
+    if(length(dropFeatures) > 0) measurements <- measurements[, setdiff(1:ncol(measurements), dropFeatures)]
+  }
+  
+  
   # Names are on both the covariates and outcome. Ensure consistent ordering.
   if(!is.null(rownames(measurements)) && !is.null(names(outcome)))
       measurements <- measurements[match(names(outcome), rownames(measurements)), ]
@@ -268,7 +296,7 @@ setMethod("prepareData", "MultiAssayExperiment",
   # Get all desired measurements tables and clinical columns.
   # These form the independent variables to be used for making predictions with.
   # Variable names will have names like RNA_BRAF for traceability.
-  dataTable <- MultiAssayExperiment::wideFormat(measurements, colDataCols = union(useFeatures, outcomeColumns))
+  dataTable <- MultiAssayExperiment::wideFormat(measurements, colDataCols = union(useFeatures[["clinical"]], outcomeColumns), check.names = FALSE)
   rownames(dataTable) <- dataTable[, "primary"]
   S4Vectors::mcols(dataTable)[, "sourceName"] <- gsub("colDataCols", "clinical", S4Vectors::mcols(dataTable)[, "sourceName"])
   dataTable <- dataTable[, -match("primary", colnames(dataTable))]
